/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/

#include <QApplication>
#include "basicInterface3Dbisthorax.h"
#include "props.h"
#include <PoLAR/Image.h>
#include <PoLAR/Util.h>
#include <osg/Geode>
#include <osg/MatrixTransform>
#include <osg/Geometry>
#include <osg/ShapeDrawable>
#include <osg/StateSet>
#include <osg/ColorMask>
#include <osg/PolygonMode>
#include <osgDB/ReadFile>
#include <osgDB/WriteFile>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/Optimizer>
#include <osg/Texture2D>
#include <iostream>
#include <PoLAR/FrameAxis.h>


int main(int argc,char ** argv)
{
    QApplication app(argc,argv);
    Interface inter;
    osg::ref_ptr<PoLAR::Image_uc> myImage;


    // Charge l'image de fond

    if(PoLAR::
            Util::fileExists("../Examples/data/img1.png"))
        myImage = new PoLAR::Image_uc("../Examples/data/img1.png", true);
    else
    {
        std::cerr << "../Examples/data/img1.png" << ": file not found" << std::endl;
        exit(0);
    }

    // Ajoute l'image chargée au fond
    inter.addImage(myImage, 1);
    double width = myImage->getWidth();
    double height = myImage->getHeight();
    inter.getViewer()->resize(width,height);
    PoLAR::Viewer::ProjectionType pt=PoLAR::Viewer2D3D::ANGIO;



    // Charge la matrice de projection
    osg::Matrixd P = PoLAR::Util::readProjectionMatrix("../Examples/data/projection.txt");
    inter.setProjection(P, pt);




    // Ajout des axes ( rouge-> X, vert-> y et bleu-> z)
    PoLAR::FrameAxis *polarAxis= new PoLAR::FrameAxis;
    polarAxis->setLightingOn();
    PoLAR::Object3D *axis = new PoLAR::Object3D(polarAxis,true);
    axis->setName("FrameAxis");
    axis->setEditOn();
    axis->translate(0,0,0);
    axis->scale(100.0f);
    inter.addNode(axis);







    // Ajout des poumons
    if(!PoLAR::Util::fileExists("../Examples/data/lung2.stl"))
    {
        std::cerr << "Unable to load model" << std::endl;
        exit(0);
    }
    else
    {

        osg::ref_ptr<PoLAR::Object3D> lung = new PoLAR::Object3D("../Examples/data/lung2.stl", true, true, true);
        osgUtil::SmoothingVisitor smooth;
        osgUtil::Optimizer optimizer;
        optimizer.optimize(lung.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
        lung->accept(smooth);
        lung->setName("lung");
        //lung->translate(-300,-500,-100); //Le problème est que quand on essai de le monter il devient plus petit
        inter.addNode(lung);
        inter.getViewer()->setTrackNode(lung.get()); // set this object to be tracked by the manipulator






    }



    // Ajout de la colonne
    if(!PoLAR::Util::fileExists("../Examples/data/spine2.stl"))
    {
        std::cerr << "Unable to load model" << std::endl;
        exit(0);
    }
    else
    {

        osg::ref_ptr<PoLAR::Object3D> spine = new PoLAR::Object3D("../Examples/data/spine2.stl", true, true, true);
        osgUtil::SmoothingVisitor smooth;
        osgUtil::Optimizer optimizer;
        optimizer.optimize(spine.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
        spine->accept(smooth);
        spine->setName("spine");
        inter.addNode(spine);
        inter.getViewer()->setTrackNode(spine.get()); // set this object to be tracked by the manipulator
    }


    // Ajout d'une côte
    if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib3L2.stl"))
    {
        std::cerr << "Unable to load model" << std::endl;
        exit(0);
    }
    else
    {

        osg::ref_ptr<PoLAR::Object3D> rib3L = new PoLAR::Object3D("../Examples/data/ribs/rib3L2.stl", true, true, true);
        osgUtil::SmoothingVisitor smooth;
        osgUtil::Optimizer optimizer;
        optimizer.optimize(rib3L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
        rib3L->accept(smooth);
        rib3L->setName("rib3L");
        inter.addNode(rib3L);
        //rib3L->rotate(M_PI/2, 0,1,0);
        inter.getViewer()->setTrackNode(rib3L.get()); // set this object to be tracked by the manipulator

    }


        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib3R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib3R = new PoLAR::Object3D("../Examples/data/ribs/rib3R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib3R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib3R->accept(smooth);
            rib3R->setName("rib3R");
            inter.addNode(rib3R);
            inter.getViewer()->setTrackNode(rib3R.get()); // set this object to be tracked by the manipulator

        }

        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib4L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib4L = new PoLAR::Object3D("../Examples/data/ribs/rib4L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib4L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib4L->accept(smooth);
            rib4L->setName("rib4L");
            inter.addNode(rib4L);
            inter.getViewer()->setTrackNode(rib4L.get()); // set this object to be tracked by the manipulator

        }



        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib4R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib4R = new PoLAR::Object3D("../Examples/data/ribs/rib4R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib4R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib4R->accept(smooth);
            rib4R->setName("rib4R");
            inter.addNode(rib4R);
            inter.getViewer()->setTrackNode(rib4R.get()); // set this object to be tracked by the manipulator

        }


        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib5L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib5L = new PoLAR::Object3D("../Examples/data/ribs/rib5L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib5L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib5L->accept(smooth);
            rib5L->setName("rib5L");
            inter.addNode(rib5L);
            inter.getViewer()->setTrackNode(rib5L.get()); // set this object to be tracked by the manipulator

        }


        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib5R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib5R = new PoLAR::Object3D("../Examples/data/ribs/rib5R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib5R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib5R->accept(smooth);
            rib5R->setName("rib5R");
            inter.addNode(rib5R);
            inter.getViewer()->setTrackNode(rib5R.get()); // set this object to be tracked by the manipulator

        }


        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib6L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib6L = new PoLAR::Object3D("../Examples/data/ribs/rib6L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib6L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib6L->accept(smooth);
            rib6L->setName("rib6L");

            inter.addNode(rib6L);
            inter.getViewer()->setTrackNode(rib6L.get()); // set this object to be tracked by the manipulator

        }


        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib6R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib6R = new PoLAR::Object3D("../Examples/data/ribs/rib6R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib6R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib6R->accept(smooth);
            rib6R->setName("rib6R");
            inter.addNode(rib6R);
            inter.getViewer()->setTrackNode(rib6R.get()); // set this object to be tracked by the manipulator

        }



        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib7L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib7L = new PoLAR::Object3D("../Examples/data/ribs/rib7L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib7L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib7L->accept(smooth);
            rib7L->setName("rib7L");
            inter.addNode(rib7L);
            inter.getViewer()->setTrackNode(rib7L.get()); // set this object to be tracked by the manipulator

        }




        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib7R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib7R = new PoLAR::Object3D("../Examples/data/ribs/rib7R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib7R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib7R->accept(smooth);
            rib7R->setName("rib7R");
            inter.addNode(rib7R);
            inter.getViewer()->setTrackNode(rib7R.get()); // set this object to be tracked by the manipulator

        }

        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib8L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib8L = new PoLAR::Object3D("../Examples/data/ribs/rib8L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib8L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib8L->accept(smooth);
            rib8L->setName("rib8L");
            inter.addNode(rib8L);
            inter.getViewer()->setTrackNode(rib8L.get()); // set this object to be tracked by the manipulator

        }

        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib8R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib8R = new PoLAR::Object3D("../Examples/data/ribs/rib8R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib8R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib8R->accept(smooth);
            rib8R->setName("rib8R");
            inter.addNode(rib8R);
            inter.getViewer()->setTrackNode(rib8R.get()); // set this object to be tracked by the manipulator

        }

        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib9L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib9L = new PoLAR::Object3D("../Examples/data/ribs/rib9L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib9L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib9L->accept(smooth);
            rib9L->setName("rib9L");
            inter.addNode(rib9L);
            inter.getViewer()->setTrackNode(rib9L.get()); // set this object to be tracked by the manipulator

        }



        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib9R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib9R = new PoLAR::Object3D("../Examples/data/ribs/rib9R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib9R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib9R->accept(smooth);
            rib9R->setName("rib9R");
            inter.addNode(rib9R);
            inter.getViewer()->setTrackNode(rib9R.get()); // set this object to be tracked by the manipulator

        }


        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib10L2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib10L = new PoLAR::Object3D("../Examples/data/ribs/rib10L2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib10L.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib10L->accept(smooth);
            rib10L->setName("rib10L");
            inter.addNode(rib10L);
            inter.getViewer()->setTrackNode(rib10L.get()); // set this object to be tracked by the manipulator

        }

        // Ajout d'une côte
        if(!PoLAR::Util::fileExists("../Examples/data/ribs/rib10R2.stl"))
        {
            std::cerr << "Unable to load model" << std::endl;
            exit(0);
        }
        else
        {

            osg::ref_ptr<PoLAR::Object3D> rib10R = new PoLAR::Object3D("../Examples/data/ribs/rib10R2.stl", true, true, true);
            osgUtil::SmoothingVisitor smooth;
            osgUtil::Optimizer optimizer;
            optimizer.optimize(rib10R.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
            rib10R->accept(smooth);
            rib10R->setName("rib10R");
            inter.addNode(rib10R);
            inter.getViewer()->setTrackNode(rib10R.get()); // set this object to be tracked by the manipulator

        }




    inter.getViewer()->center();
    inter.show();



    return app.exec();
}
